<?php

/**
 * Get instances for bundle by field type
 * 
 * @param string $field_type
 *   Field type.
 * @param string $entity_type
 *   Entity type.
 * @param string $bundle_name
 *   Bundle name.
 * @return array
 *   Array of instances arrays.
 */
function hotspots_get_instances_for_bundle_by_field_type($field_type, $entity_type, $bundle_name) {
  $field_info_fields = field_info_field_by_ids();
  $field_info_instances = field_info_instances($entity_type, $bundle_name);

  $instances = array();
  foreach ($field_info_instances as $field_info_instance) {
    if ($field_info_fields[$field_info_instance['field_id']]['type'] == $field_type) {
      $instances[] = $field_info_instance;
    };
  }
  return $instances;
}

/**
 * Determine existing image instance in node for image by file id.
 * If instance exists, hotspot data will be added to node object.
 * 
 * @param object $node
 *   Node object with image.
 * @param numeric $fid
 *   File ID of image.
 * @return boolean
 *   TRUE, if image instance exists, otherwise ELSE.
 */
function hotspots_determine_image_instance_for_image(&$node, $fid) {
  $image_instances = hotspots_get_instances_for_bundle_by_field_type('image', 'node', $node->type);
  foreach ($image_instances as $image_instance) {
    $image_instance_items = field_get_items('node', $node, $image_instance['field_name']);
    foreach($image_instance_items as $image_instance_item) {
      if ($image_instance_item['fid'] == $fid) {
        $node->hotspot_data = _hotspots_creation_hotspot_data($node, $image_instance, $image_instance_item);
        return TRUE;
      }
    }
  }
  return FALSE;
}

/**
 * Helper function for creating hotspot data.
 *
 * @param array $image_instance
 *   Image instance for which we creating hotspot data.
 * @return array
 *   Array of hotspot data.
 * 
 * @todo: describe hotspot data array in description.
 */
function _hotspots_creation_hotspot_data($node, $image_instance, $image_instance_item) {
  $hotspot_instances = hotspots_get_instances_for_bundle_by_field_type('hotspot', $image_instance['entity_type'], $image_instance['bundle']);
  $hotspot_instance = reset($hotspot_instances);
  $hotspot_instance_items = field_get_items($image_instance['entity_type'], $node, $hotspot_instance['field_name']);
  if (empty($hotspot_instance_items)) {
    $hotspot_instance_items = array();
  }

  $hotspot_data = array(
    'image_instance' => $image_instance,
    'image_instance_item' => $image_instance_item,
    'hotspot_instance' => $hotspot_instance,
    'hotspot_instance_items' => $hotspot_instance_items,
  );
  return $hotspot_data;
}