<?php

/**
 * Implements hook_field_formatter_info().
 */
function hotspots_field_formatter_info() {
  $items = array();
  
  $image_items = image_field_formatter_info();
  $items['hotspots_image'] = array(
    'label' => t('Image with hotspots'), 
    'field types' => array('image'),
    'settings' => $image_items['image']['settings'],
  );
  return $items;
}

/**
 * Implements hook_field_formatter_settings_form().
 */
function hotspots_field_formatter_settings_form($field, $instance, $view_mode, $form, &$form_state) {
  return image_field_formatter_settings_form($field, $instance, $view_mode, $form, $form_state);
}

/**
 * Implements hook_field_formatter_settings_summary().
 */
function hotspots_field_formatter_settings_summary($field, $instance, $view_mode) {
  return image_field_formatter_settings_summary($field, $instance, $view_mode);
}

/**
 * Implements hook_field_formatter_view().
 */
function hotspots_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $image_display = $display;
  $image_display['type'] = 'image';
  $image_display['module'] = 'image';
  $image_display['hotspots'] = TRUE;
  $elements = image_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $image_display);
  
  $hotspots_instances = hotspots_get_instances_for_bundle_by_field_type('hotspot', $entity_type, $entity->type);
  
  if (empty($hotspots_instances)) {
    return $elements;
  }
  
  // @todo: replace in #attach.
  drupal_add_css(drupal_get_path('module', 'hotspots') . '/theme/css/hotspots-style.css');
  drupal_add_css(drupal_get_path('module', 'hotspots') . '/theme/css/hotspots-ie.css', array('browsers' => array('IE' => 'lt IE 9', '!IE' => FALSE)));
  
  $hotspots_field_name = $hotspots_instances[0]['field_name'];
  $hotspot_items = (array)field_get_items($entity_type, $entity, $hotspots_field_name);
  foreach ($elements as &$element) {
    // Get image and it's attributes
    $image = $element;
    $rendered_image = render($image);
    
    $image_fid = $image['#item']['fid'];
    if (!empty($image['#image_style'])) {
      $image_path = image_style_url($image['#image_style'], $image['#item']['uri']);
    }
    else {
      $image_path = file_create_url($image['#item']['uri']);
    }
    $image_size = getimagesize($image_path);
    $image_width = $image_size[0];
    $image_height = $image_size[1];
    
    // Get overlay
    $overlay_styles = array(
      'width' => $image_width . 'px',
      'height' => $image_height . 'px',
    );
    
    $overlay = array(
      '#theme' => 'html_tag',
      '#tag' => 'div',
      '#value' => '',
      '#attributes' => array(
        'class' => array('overlay'),
        'style' => _hotspots_format_styles($overlay_styles),
      ),
    );
    
    // Get hotspots and hotspots_pointers
    $hotspots = array();
    $hotspots_pointers = array(
      '#theme' => 'hotspots_hotspots_pointers',
      '#hotspots_pointers' => array(),
    );
    $element = array();
    $hotspot_index = 0;
    foreach ($hotspot_items as $hotspot_item) {
      $hotspot_fid = $hotspot_item['fid'];
      if ($image_fid == $hotspot_fid) {
        
        // Hotspots
        $hotspot_styles = array(
          'left' => round($image_width * $hotspot_item['coords_x1']) . 'px',
          'top' => round($image_height * $hotspot_item['coords_y1']) . 'px',
          'width' => round($image_width * ($hotspot_item['coords_x2'] - $hotspot_item['coords_x1'])) . 'px',
          'height' => round($image_height * ($hotspot_item['coords_y2'] - $hotspot_item['coords_y1'])) . 'px',
        );
        $hotspot_image_styles = array(
          'left' => '-' . $hotspot_styles['left'],
          'top' => '-' . $hotspot_styles['top'],
          'width' => $image_width . 'px',
          'height' => $image_height . 'px',
        );
        
        $hotspot_image = array(
          '#theme' => 'image',
          '#path' => $image_path,
          '#width' => $image_width,
          '#height' => $image_height,
          '#alt' => $image['#item']['alt'],
          '#title' => $image['#item']['title'],
          '#attributes' => array(
            'style' => _hotspots_format_styles($hotspot_image_styles),
          ),
        );
        
        $hotspots[] = array(
          '#theme' => 'hotspots_hotspot',
          '#hotspot_styles' => _hotspots_format_styles($hotspot_styles),
          '#hotspot_index' => $hotspot_index,
          '#image' => render($hotspot_image),
          '#hotspot_type' => 'view',
        );
        
        // Pointers
        $hidden_fields = array();
        
        $hidden_fields['hotspot_index'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_index,
          '#attributes' => array('class' => 'index',),
        );
        $hidden_fields['hotspot_title'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['title'],
          '#attributes' => array('class' => 'title',),
        );
        $hidden_fields['hotspot_value'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['value'],
          '#attributes' => array('class' => 'value',),
        );
        $hidden_fields['coords_x1'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['coords_x1'],
          '#attributes' => array('class' => 'coords-x1',),
        );
        $hidden_fields['coords_y1'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['coords_y1'],
          '#attributes' => array('class' => 'coords-y1',),
        );
        $hidden_fields['coords_x2'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['coords_x2'],
          '#attributes' => array('class' => 'coords-x2',),
        );
        $hidden_fields['coords_y2'] = array(
          '#type' => 'hidden',
          '#value' => $hotspot_item['coords_y2'],
          '#attributes' => array('class' => 'coords-y2',),
        );
        
        $hotspots_pointers['#hotspots_pointers'][] = array(
          '#theme' => 'hotspots_hotspot_pointer',
          '#hotspot_title' => $hotspot_item['title'],
          '#hotspot_index' => $hotspot_index,
          '#hidden_field_hotspot_index' => render($hidden_fields),
        );
        
        // Incriment index
        $hotspot_index++;
      };
    };
    
    foreach ($hotspots_pointers['#hotspots_pointers'] as $index => $hotspot_pointer) {
      $hotspots_pointers['#hotspots_pointers'][$index] = render($hotspot_pointer);
    }
    $hotspots_pointers = render($hotspots_pointers);
    
    $element = array(
      '#theme' => 'hotspots_view_page',
      '#image' => $rendered_image,
      '#overlay' => render($overlay),
      '#hotspots' => render($hotspots),
      '#hotspots_pointers' => $hotspots_pointers,
      '#attached' => array(
        'js' => array(
          drupal_get_path('module', 'hotspots') . '/includes/hotspots.script.js',
        ),
      ),
    );
  };
  return $elements;
}
