<?php

/**
 * Implements hook_field_widget_info().
 */
function hotspots_field_widget_info() {
  return array(
    'hotspot_default' => array(
      'label' => t('Hotspot'),
      'field types' => array('hotspot'),
      'behaviors' => array(
        'multiple values' => FIELD_BEHAVIOR_DEFAULT,
        'default value' => FIELD_BEHAVIOR_DEFAULT,
      ),
    ),
  );
}

/**
 * Implements hook_field_widget_form().
 */
function hotspots_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  // Hotspot field form
  $element = array();
  if ($instance['widget']['type'] == 'hotspot_default') {
    // Exit if hotspot data don't exist.
    $isset_hotspot_data = isset($form_state['node']->hotspot_data);
    if (!$isset_hotspot_data) {
      return FALSE;
    }
    
    // Variables definition
    $hotspot_data = $form_state['node']->hotspot_data;
    $hotspot_field_name = $hotspot_data['hotspot_instance']['field_name'];
    $hotspot_image_fid = $hotspot_data['image_instance_item']['fid'];
    $values = isset($items[$delta]) ? $items[$delta] : array();
    $new_hotspot = empty($values);
    $current_image_hotspot = !$new_hotspot && $values['fid'] == $hotspot_image_fid;
    $show_field = $new_hotspot || $current_image_hotspot;
    
    if (isset($form_state['values'][$hotspot_field_name][$langcode][$delta])) {
      $values = $form_state['values'][$hotspot_field_name][$langcode][$delta];
    } 
    
    // Hotspot fields
    if ($show_field) {
      $element['#prefix'] = '<div class="hotspot-item ' . (isset($values['status']) ? $values['status'] : 'collapsed') . '">';
      $element['#suffix'] = '</div>';
    }
    if ($show_field) {
      $element['title_label'] = array(
        '#theme' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => array(
          'class' => array('title-label'),
        ),
        '#value' => (isset($values['title']) && !empty($values['title'])) ? $values['title'] : t('New hotspot'),
      );
    }
    $element['title'] = array(
      '#type' => (($show_field) ? 'textfield' : 'value'),
      '#title' => t('Name'),
      '#default_value' => isset($values['title']) ? $values['title'] : '',
      '#attributes' => array('class' => array('title',),),
    );
    $element['value'] = array(
      '#type' => (($show_field) ? 'textfield' : 'value'),
      '#title' => t('Value'),
      '#default_value' => isset($values['value']) ? $values['value'] : '',
      '#attributes' => array('class' => array('value',),),
    );
    if ($show_field) {
      $element['submit'] = array(
        '#theme' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => array(
          'class' => array('submit'),
        ),
        '#value' => t('Ok'),
      );
    }
    $element['fid'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => isset($values['fid']) ? $values['fid'] : '',
      '#attributes' => array('class' => 'fid',),
    );
    $element['coords_x1'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => isset($values['coords_x1']) ? $values['coords_x1'] : '',
      '#attributes' => array('class' => 'coords-x1',),
    );
    $element['coords_y1'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => isset($values['coords_y1']) ? $values['coords_y1'] : '',
      '#attributes' => array('class' => 'coords-y1',),
    );
    $element['coords_x2'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => isset($values['coords_x2']) ? $values['coords_x2'] : '',
      '#attributes' => array('class' => 'coords-x2',),
    );
    $element['coords_y2'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => isset($values['coords_y2']) ? $values['coords_y2'] : '',
      '#attributes' => array('class' => 'coords-y2',),
    );
    $element['status'] = array(
      '#type' => (($show_field) ? 'hidden' : 'value'),
      '#default_value' => 'collapsed',
      '#attributes' => array('class' => 'status',),
    );
  }
  return $element;
}

/**
 * Implements hook_field_widget_form_alter().
 */
function hotspots_field_widget_form_alter(&$element, &$form_state, $context) {
  hotspots_altering_image_field_form($element, $form_state, $context);
}

/**
 * Auxiliary function for hotspots_field_widget_form_alter.
 * Altering edit image field form, which adds link to hotspot edit page.
 * 
 * @see function hotspots_image_field_widget_process.
 */
function hotspots_altering_image_field_form(&$element, &$form_state, $context) {
  if ($context['field']['type'] != 'image'){
    return;
  }
  
  $node = $form_state['node'];
  if (!isset($node->nid)) {
    return;
  }
  
  if (!isset($element['#field_name']) && !isset($element[0]['#field_name'])) {
    return;
  }

  $field_name = isset($element['#field_name']) ? $element['#field_name'] : $element[0]['#field_name'];
  $items = field_get_items('node', $node, $field_name);
  foreach(element_children($element) as $item_key) {
    if (isset($items[$item_key]) && !empty($element[$item_key]['#default_value']['fid'])) {
      $element[$item_key]['#process'][]  = 'hotspots_altering_image_field_form_process';
    }
  }
}

/**
 * Process handler.
 */
function hotspots_altering_image_field_form_process($element, &$form_state, $form) {
  $path = array();
  $path[] = $element['#entity_type'];
  $path[] = $form_state['node']->nid;
  $path[] = 'hotspots';
  $path[] = 'fid';
  $path[] = $element['#file']->fid;
  $path = implode('/', $path);
  
  $element['hotspot_link'] = array(
    '#theme' => 'link',
    '#text' => t('Edit hotspots'),
    '#path' => $path,
    '#options' => array(
      'attributes' => array(),
      'html' => FALSE,
    ),
  );
  return $element;
}
