<?php

/**
 * Form alter extension.
 * Hotspots field form.
 */
function hotspots_hotspots_field_form_node(&$form, &$form_state, $form_id) {
  // Define variables block.
  $node = $form_state['node'];
  $hotspot_data = $node->hotspot_data;
  $hotspot_field_name = $hotspot_data['hotspot_instance']['field_name'];
  $requires_fields = array('form_id', 'form_token', 'changed', 'form_build_id', 'actions', $hotspot_field_name);
  
  // Remove all fields except hotspot fields.
  foreach (element_children($form) as $field_name) {
    if (!in_array($field_name, $requires_fields)) {
      $form[$field_name]['#access'] = FALSE;
    }
  }
  
  // Hotspot image
  $form['hotspot_image'] = hotspots_edit_hotspots_page_form_image($node, $hotspot_data);
  
  // Hotspot field
  $form[$hotspot_field_name] = hotspots_edit_hotspots_page_form_hotspots_field($form[$hotspot_field_name]);
  
  // Add submit
  $form['actions']['submit']['#submit'][] = 'hotspots_form_node_form_alter_submit';
}

/**
 * Submit handler.
 */
function hotspots_form_node_form_alter_submit($form, &$form_state) {
  $node = $form_state['node'];
  $form_state['redirect'] = 'node/' . $node->nid . '/edit';
}

/**
 * Hotspot image building form.
 * 
 * @param object $node
 *   Image node.
 * @param array $hotspot_data
 *   Hotspot data array.
 * @return array 
 *   The hotspot image form.
 */
function hotspots_edit_hotspots_page_form_image($node, $hotspot_data) {
  $image = array(
    '#prefix' => '<div class="hotspots-edit"><div class="image">',
    '#suffix' => '</div></div>',
    '#weight' => 0,
  );
  $image['image'] = field_view_value($hotspot_data['image_instance']['entity_type'], $node, $hotspot_data['image_instance']['field_name'], $hotspot_data['image_instance_item']);
  $image['image']['#image_style'] = 'hotspots_default';
  $image['fid'] = array(
    '#type' => 'hidden', 
    '#value' => $image['image']['#item']['fid'],
    '#attributes' => array('class' => array('image-fid')),
  );
  $image['hotspots'] = array(
    '#prefix' => '<div class="hotspots">',
    '#suffix' => '</div>',
  );
  $image['hotspot_popup'] = array(
    '#prefix' => '<div class="hotspots-popup">',
    '#suffix' => '</div>',
  );
  $hotspot_index = 0;
  // Obtain hotspots
  foreach ($hotspot_data['hotspot_instance_items'] as $hotspot_item) {
    if ($hotspot_item['fid'] == $hotspot_data['image_instance_item']['fid']) {
      $styles = _hotspots_get_hotspot_styles($image['image']['#item'], $hotspot_item);
      $image['hotspots'][] = array(
        '#type' => 'markup',
        '#markup' => theme('hotspots_hotspot', array('hotspot_styles' => $styles, 'hotspot_index' => $hotspot_index, 'hotspot_type' => 'edit',)),
      );
      $hotspot_index++;
    }
  }
  return $image;
}

/**
 * Hotspots field rebuilding form.
 *
 * @param array $hotspot_field
 *   The existing hotspot field form.
 * @return array
 *   The rebuilded hotspot field form.
 */
function hotspots_edit_hotspots_page_form_hotspots_field($hotspot_field) {

  $hotspot_field['#prefix'] = '<div class="hotspots-field">';
  $hotspot_field['#suffix'] = '</div>';
  $hotspot_field['#weight'] = 1;
  $hotspot_field['#attached'] = array(
    'js' => array(
      drupal_get_path('module', 'hotspots') . '/includes/jquery.Jcrop.min.js',
      drupal_get_path('module', 'hotspots') . '/includes/hotspots.script.js',
    ),
    'css' => array(
      drupal_get_path('module', 'hotspots') . '/theme/css/hotspots-style.css',
      drupal_get_path('module', 'hotspots') . '/theme/css/hotspots-ie.css' => array('browsers' => array('IE' => 'lt IE 9', '!IE' => FALSE)),
    ),
  );
  
  $hotspot_items = &$hotspot_field[$hotspot_field['#language']];
  // Remove theme field_multiple_value_form.
  $hotspot_items['#theme'] = NULL;
  // Remove weight fields from hotspot fields.
  foreach (element_children($hotspot_items) as $hotspot_key) {
    if (isset($hotspot_items[$hotspot_key]['_weight'])) {
      $hotspot_items[$hotspot_key]['_weight']['#access'] = FALSE;
    }
  }
  
  return $hotspot_field;
}

/**
 * Helper function for calculation styles.
 * 
 * @param array $image
 *   Image instance item.
 * @param array $hotspot_item
 *   Hotspot data array.
 * @return string
 *   Inline styles string. 
 */
function _hotspots_get_hotspot_styles($image, $hotspot_item) {
  // Math calculation.
  $orig_width = $image['width'];
  $orig_height = $image['height'];
  $thumb_width = HOTSPOT_PRESET_WIDTH;
  $thumb_height = ($orig_height / $orig_width) * $thumb_width;
  $coords_x1 = round($thumb_width * $hotspot_item['coords_x1']);
  $coords_y1 = round($thumb_height * $hotspot_item['coords_y1']);
  $coords_x2 = round($thumb_width * $hotspot_item['coords_x2']);
  $coords_y2 = round($thumb_height * $hotspot_item['coords_y2']);
  $hotspot_left = $coords_x1;
  $hotspot_top = $coords_y1;
  $hotspot_width = $coords_x2 - $coords_x1;
  $hotspot_height = $coords_y2 - $coords_y1;
  
  // Style creation.
  $style = array();
  $style[] = 'top: ' . $hotspot_top . 'px';
  $style[] = 'left: ' . $hotspot_left . 'px';
  $style[] = 'width: ' . $hotspot_width . 'px';
  $style[] = 'height: ' . $hotspot_height . 'px';
  return implode('; ', $style);
}